package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesYfReportDao;
import cn.estsh.i3plus.ext.mes.pojo.model.MesShippingOrderManagementDetailModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportResultModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.report.MesGeneralExportReportModel;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description : 报表-数据方法实现
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/15 13:30
 * @Modify:
 **/
@Service
public class MesYfReportDaoImpl implements IMesYfReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesDeviceDataQueryReportResultModel> queryMesDeviceDataByPager(MesDeviceDataQueryReportConditionModel queryReportModel, Pager pager) {
        StringBuilder dataQuerySql = new StringBuilder();
        StringBuilder countQuerySql = new StringBuilder();
        StringBuilder commonQuerySql = new StringBuilder();
        dataQuerySql.append("select new ")
                .append(MesDeviceDataQueryReportResultModel.class.getName())
                .append("(organizeCode, workCenterCode, workCellCode, workOrderNo, serialNumber, ")
                .append("logType, message, equipmentCode, equipmentName, createDatetime, createUser, " +
                        "modifyDatetime, modifyUser) ");
        countQuerySql.append("select count(1) ");
        commonQuerySql.append("from MesWorkCellScanMonitorLog " +
                "where isValid = :isValid and isDeleted = :isDeleted and organizeCode = :organizeCode " +
                "and createDatetime >= :createDateTimeStart and createDatetime <= :createDateTimeEnd ");
        //非空查询条件拼接语句
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            commonQuerySql.append("and workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            commonQuerySql.append("and workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            commonQuerySql.append("and workOrderNo = :workOrderNo ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            commonQuerySql.append("and serialNumber = :serialNumber ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            commonQuerySql.append("and equipmentCode = :equipmentCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            commonQuerySql.append("and logType = :logType ");
        }
        Query dataQuery = entityManager.createQuery(dataQuerySql.append(commonQuerySql).toString(), MesDeviceDataQueryReportResultModel.class);
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        dataQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        dataQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        Query countQuery = entityManager.createQuery(countQuerySql.append(commonQuerySql).toString());
        countQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        countQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        countQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        countQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        countQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            dataQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
            countQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            dataQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
            countQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            dataQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
            countQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            dataQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
            countQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            dataQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
            countQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            dataQuery.setParameter("logType", queryReportModel.getLogType());
            countQuery.setParameter("logType", queryReportModel.getLogType());
        }
        //本次查询满足条件的总数
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        //查询本页数据
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<MesDeviceDataQueryReportResultModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

    @Override
    public ListPager<MesGeneralExportReportModel> queryGeneralExport(MesGeneralExportReportModel mesGeneralExportReportModel, Pager pager) {
        StringBuilder dataHql = new StringBuilder();
        dataHql.append(" select new ").append(MesGeneralExportReportModel.class.getName()).append("(workOrder.organizeCode,workOrder.workCenterCode,workOrder.id,workOrder.workOrderNo,workOrder.custSn,partType.partTypeName,assembly.assemblySn,orderAssembly.createDatetime,workOrder.shippingTime)");
        //共用HQL
        String commonHql = getCommonHql(mesGeneralExportReportModel);
        //行数
        Query countQuery = entityManager.createQuery("select count(1) " + commonHql);
        setParameter(mesGeneralExportReportModel, countQuery);
        //数据
        Query dataQuery = entityManager.createQuery(dataHql.append(commonHql).append(getOrderBy(mesGeneralExportReportModel)).toString(), MesShippingOrderManagementDetailModel.class);
        setParameter(mesGeneralExportReportModel, dataQuery);
        if (Objects.isNull(pager)) return new ListPager<>(dataQuery.getResultList(), pager);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        if (count <= 0) return new ListPager<>(new ArrayList<>(), pager);
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(dataQuery.getResultList(), pager);
    }

    private void setParameter(MesGeneralExportReportModel model, Query query) {
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("organizeCode", model.getOrganizeCode());
        query.setParameter("workOrderStatus", Stream.of(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue(),MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue(),MesExtEnumUtil.ORDER_STATUS.SHIPPING.getValue()).collect(Collectors.toList()));
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) query.setParameter("shippingTimeStart", model.getShippingTimeStart());
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) query.setParameter("shippingTimeEnd", model.getShippingTimeEnd());
    }

    private String getOrderBy(MesGeneralExportReportModel model) {
        StringBuilder orderBy = new StringBuilder();
        if (StringUtils.isEmpty(model.getOrderByParam()) || StringUtils.isEmpty(model.getAscOrDesc())) {
            model.setOrderByParam("createDatetime");
            model.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        }
        return orderBy.append(" order by d.").append(model.getOrderByParam()).append(" ").append(CommonEnumUtil.ASC_OR_DESC.DESC.getValue() == model.getAscOrDesc() ? "desc" : "asc").toString();
    }

    private String getCommonHql(MesGeneralExportReportModel model) {
        StringBuilder commonHql = new StringBuilder();
        commonHql.append(" from MesWorkOrderAssembly orderAssembly left join MesWorkOrder workOrder on orderAssembly.workOrderNo = workOrder.workOrderNo and workOrder.organizeCode = :organizeCode and workOrder.isDeleted=:isDeleted and workOrder.isValid=:isValid");
        commonHql.append(" left join MesProductionAssembly assembly on workOrder.custSn = assembly.custSn and orderAssembly.organizeCode=:organizeCode and orderAssembly.isDeleted=:isDeleted and orderAssembly.isValid=:isValid ");
        commonHql.append(" left join MesProdRuleSortCfg sortCfg on orderAssembly.pid = sortCfg.id and sortCfg.organizeCode = :organizeCode and sortCfg.isDeleted=:isDeleted and sortCfg.isValid=:isValid");
        commonHql.append(" left join MesPartType partType on sortCfg.partTypeCode = partType.partTypeCode and partType.organizeCode = :organizeCode and partType.isDeleted=:isDeleted and partType.isValid=:isValid");
        commonHql.append(" where orderAssembly.organizeCode=:organizeCode and orderAssembly.isDeleted=:isDeleted and orderAssembly.isValid=:isValid and workOrder.organizeCode = :organizeCode");
        commonHql.append(" and workOrder.isDeleted=:isDeleted and workOrder.isValid=:isValid and assembly.organizeCode = :organizeCode and assembly.isDeleted=:isDeleted and assembly.isValid=:isValid");
        commonHql.append(" and sortCfg.organizeCode = :organizeCode and sortCfg.isDeleted=:isDeleted and sortCfg.isValid=:isValid and partType.organizeCode = :organizeCode and partType.isDeleted=:isDeleted and partType.isValid=:isValid ");
        commonHql.append(" and partType.assemblyDescription = '通用追溯' and workOrder.workOrderStatus in (:workOrderStatus) ");
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) commonHql.append(" and workOrder.workCenterCode like '%").append(model.getWorkCenterCode()).append("%' ");
        if (!StringUtils.isEmpty(model.getCustSn())) commonHql.append(" and orderAssembly.custSn like '%").append(model.getCustSn()).append("%' ");
        if (!StringUtils.isEmpty(model.getAssemblySn())) commonHql.append(" and orderAssembly.assemblySn like '%").append(model.getAssemblySn()).append("%' ");
        if (!StringUtils.isEmpty(model.getWorkOrderNo())) commonHql.append(" and workOrder.workOrderNo like '%").append(model.getWorkOrderNo()).append("%' ");
        if (!StringUtils.isEmpty(model.getShippingTimeStart())) commonHql.append(" and workOrder.shippingTime >= :shippingTimeStart ");
        if (!StringUtils.isEmpty(model.getShippingTimeEnd())) commonHql.append(" and workOrder.shippingTime <= :shippingTimeEnd ");
        return commonHql.toString();
    }

}
